local PartyMembers = { }
local PartyMemberClassIds = { }
local IsPartyMembersInitialized = false
local PlayerName = UnitName("player")
local IsPlayerInRaid = (UnitInRaid("player") ~= nil)
local IsPlayerInBg = ((UnitInBattleground("player") or 0) > 0)

local function GetPlayerRaidSubGroup()
	if (UnitInRaid("player") == nil) then return nil end

	for index = 1, 40
	do
		local name, _, subGroup = GetRaidRosterInfo(index)	
		if (name == PlayerName) then return subGroup end
	end
	
	return nil
end

local PlayerSubGroup = GetPlayerRaidSubGroup()

local function SendChatMessage(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b)
end

local function PlayPartyChangedSound()
	PlaySoundFile("Interface/AddOns/PartyChangedNotifier/party_changed.mp3", "Master")
end

local function SendSubGroupChangedMessage(newGroup)
	PlayPartyChangedSound()
	if (newGroup ~= nil) then SendChatMessage("You are now in group " .. newGroup .. ".") end
end

local function GetUnitClassId(name)
	local raidIndex = UnitInRaid(name)
	if (raidIndex == nil) then return nil end
	
	local _, _, classId = UnitClass("raid" .. raidIndex)
	return classId
end

local ClassColors = 
{
	[1] = "FFC79C6E",
	[2] = "FFF58CBA",
	[3] = "FFABD473",
	[4] = "FFFFF569",
	[5] = "FFFFFFFF",
	[6] = "FFC41F3B",
	[7] = "FF0070DE",
	[8] = "FF69CCF0",
	[9] = "FF9482C9",
	[10] = "FF00FF96",
	[11] = "FFFF7D0A",
	[12] = "FFA330C9"
}
local function GetColoredName(name, groupN)
	local classId = GetUnitClassId(name)
	if (classId == nil) then classId = PartyMemberClassIds[name] end
	if (classId == nil) then return end
	
	local color = ClassColors[classId]
	return "|c" .. color .. name .. "|r"
end

local function SendPartyChangedMessage(oldName, newName)
	PlayPartyChangedSound()
	
	if (oldName ~= nil) then oldName = GetColoredName(oldName, raidIndex) end
	if (newName ~= nil) then newName = GetColoredName(newName, raidIndex) end

	if (oldName == nil and newName ~= nil) then SendChatMessage(newName .. " is now in your party.") end
	if (oldName ~= nil and newName == nil) then SendChatMessage(oldName .. " is no longer in your party.") end
	if (oldName ~= nil and newName ~= nil) then SendChatMessage(oldName .. " was replaced by " .. newName .. " in your party.") end
end

local function IsValueInTable(tab, searchValue)
	for key, value in pairs(tab) do
		if (value == searchValue) then return true end
	end
	
	return false
end

local function GetUnitName(groupN)
	local name, realm = UnitName("party" .. groupN)
	
	if (name == "Unknown") then return name end
	if (realm == nil or realm == "") then return name else return name .. "-" .. realm end
end

local function UpdatePartyMembers()
	for groupN = 1, 4
	do
		
		local unitName = GetUnitName(groupN)
		if (unitName == "Unknown") then return false end
		
		PartyMembers[groupN] = unitName
		if (unitName ~= nil) then PartyMemberClassIds[unitName] = GetUnitClassId(unitName) end
	end
	
	return true
end

local function GetPartyMemberNames()
	local partyMembers = { }

	for groupN = 1, 4
	do
		local name = GetUnitName(groupN)
		if (name ~= nil and name ~= "Unknown") then table.insert(partyMembers, name) end
	end
	
	return partyMembers
end

local function PartyChanged(groupN, savedName, currentName)
	if (currentName == "Unknown") then return end	
	local currentMembers = GetPartyMemberNames()
	
	--UNIT REPLACED: saved member not present in current list and new member not present in unupdated member list
	if (not IsValueInTable(currentMembers, savedName) and not IsValueInTable(PartyMembers, currentName))
	then
		return SendPartyChangedMessage(savedName, currentName)
	end
	
	--UNIT LEFT: saved member not present in party anymore, unit moved to other group or left raid
	if (savedName ~= nil and not IsValueInTable(currentMembers, savedName))
	then
		return SendPartyChangedMessage(savedName, nil)
	end

	--UNIT JOINED: current member not present in unupdated party members list but is present in current list
	if (not IsValueInTable(PartyMembers, currentName) and IsValueInTable(currentMembers, currentName))
	then
		return SendPartyChangedMessage(nil, currentName)	
	end
end

local function UpdateRaidData()
	UpdatePartyMembers()
	if (IsPlayerInRaid) then PlayerSubGroup = GetPlayerRaidSubGroup() end
	if (not IsPlayerInRaid) then IsPartyMembersInitialized = false end
end

local _timeSinceLastUpdate = 0
local function OnUpdate(self, elapsed)
    _timeSinceLastUpdate = _timeSinceLastUpdate + elapsed
    if (_timeSinceLastUpdate < 0.5) then return end
	_timeSinceLastUpdate = 0

	--return if player has just joined/returned battleground or is in one
	local isPlayerInBg = IsPlayerInBg
	IsPlayerInBg = ((UnitInBattleground("player") or 0) > 0)
	if (IsPlayerInBg ~= isPlayerInBg) then return UpdateRaidData() end
	if (IsPlayerInBg) then return end
	
	--return if player has just joined a raid or is not in one
	local isPlayerInRaid = IsPlayerInRaid
	IsPlayerInRaid = (UnitInRaid("player") ~= nil)
	if (IsPlayerInRaid ~= isPlayerInRaid) then return UpdateRaidData() end
	if (not IsPlayerInRaid) then return end

	--init party members if not yet done
	if (not IsPartyMembersInitialized) 
	then 
		IsPartyMembersInitialized = UpdatePartyMembers()
		return
	end

	--notify and then return if the player has been moved to a different group
	local playerSubGroup = PlayerSubGroup
	PlayerSubGroup = GetPlayerRaidSubGroup()
	if (PlayerSubGroup ~= playerSubGroup)
	then
		UpdateRaidData()
		SendSubGroupChangedMessage(PlayerSubGroup)
		return
	end

	--loop over party members
	for groupN = 1, 4
	do
		--retrieve names at this index
		local savedName = PartyMembers[groupN]
		local currentName = GetUnitName(groupN)
		
		--see if party member name at this index has changed
		if (currentName ~= savedName) then PartyChanged(groupN, savedName, currentName) end
	end
	
	--finally update party member list
	UpdatePartyMembers()
end

local frame = CreateFrame("Frame", nil, UIParent)
frame:SetScript("OnUpdate", OnUpdate)